@@ -24,32 +24,20 @@ const {
 class PinnedFavs {
     constructor(params) {
         this.params = params;
-        this.favoriteSettingKey = 'favorite-apps';
         this.reload();
     }
 
     reload() {
         const {state, signals, settings} = this.params;
         const appSystem = state.trigger('getAppSystem');
-        if (signals.isConnected('changed::favorite-apps', global.settings)) {
-            signals.disconnect('changed::favorite-apps', global.settings);
-        }
         if (signals.isConnected('changed::pinned-apps', settings)) {
             signals.disconnect('changed::pinned-apps', settings);
         }
         let cb = () => this.onFavoritesChange();
-        if (state.settings.systemFavorites) {
-            signals.connect(global.settings, 'changed::favorite-apps',  cb);
-        } else {
-            signals.connect(settings, 'changed::pinned-apps', cb);
-        }
+        signals.connect(settings, 'changed::pinned-apps', cb);
         this._favorites = [];
         let ids = [];
-        if (state.settings.systemFavorites) {
-            ids = global.settings.get_strv(this.favoriteSettingKey);
-        } else {
-            ids = settings.getValue('pinned-apps');
-        }
+        ids = settings.getValue('pinned-apps');
         for (let i = 0, len = ids.length; i < len; i++) {
             let refFav = findIndex(this._favorites, (item) => item.id === ids[i]);
             if (refFav === -1) {
@@ -91,37 +79,12 @@ class PinnedFavs {
                 uniqueSet.add(this._favorites[i].id);
             }
         }
-
-        if (this.params.state.settings.systemFavorites) {
-            global.settings.set_strv(this.favoriteSettingKey, ids);
-        } else {
-            this.params.settings.setValue('pinned-apps', ids);
-        }
+        this.params.settings.setValue('pinned-apps', ids);
     }
 
     onFavoritesChange() {
-        let currentAppList = this.params.state.trigger('getCurrentAppList');
-        if (!this.params.state.settings.groupApps) {
-            setTimeout(() => currentAppList.refreshList(), 0);
-            return;
-        }
-        let oldFavoritesIds = [];
-        let newFavoritesIds = [];
-        for (let i = 0; i < this._favorites.length; i++) {
-            oldFavoritesIds.push(this._favorites[i].id);
-        }
         this.reload();
-        for (let i = 0; i < this._favorites.length; i++) {
-            newFavoritesIds.push(this._favorites[i].id);
-        }
-        for (let i = 0; i < oldFavoritesIds.length; i++) {
-            if (newFavoritesIds.indexOf(oldFavoritesIds[i]) < 0) {
-                this.triggerUpdate(oldFavoritesIds[i], false);
-            }
-        }
-        for (let i = 0; i < this._favorites.length; i++) {
-            this.triggerUpdate(newFavoritesIds[i], true);
-        }
+        this.params.state.trigger('refreshAllAppLists');
     }
 
     addFavorite(opts = {appId: null, app: null, pos: -1}) {
@@ -130,9 +93,6 @@ class PinnedFavs {
         if (!opts.app) {
             opts.app = appSystem.lookup_app(opts.appId);
         }
-        if (!opts.app) {
-            opts.app = appSystem.lookup_settings_app(opts.appId);
-        }
         if (!opts.app) {
             opts.app = appSystem.lookup_desktop_wmclass(opts.appId);
         }
@@ -251,7 +211,9 @@ class GroupedWindowListApplet extends Applet.Applet {
             lastTitleDisplay: null,
             scrollActive: false,
             thumbnailMenuOpen: false,
-            thumbnailCloseButtonOffset: global.ui_scale > 1 ? -10 : 0
+            thumbnailCloseButtonOffset: global.ui_scale > 1 ? -10 : 0,
+            addingWindowToWorkspaces: false,
+            removingWindowFromWorkspaces: false,
         });
 
         // key-function pairs of actions that can be triggered from the store's callback queue. This allows the
@@ -271,11 +233,13 @@ class GroupedWindowListApplet extends Applet.Applet {
                 each(this.appLists, function(appList) {
                     appList.windowAdded(appList.metaWorkspace, win, app, isFavoriteApp);
                 });
+                this.state.addingWindowToWorkspaces = false;
             },
             removeWindowFromAllWorkspaces: (win) => {
                 each(this.appLists, function(appList) {
                     appList.windowRemoved(appList.metaWorkspace, win);
                 });
+                this.state.removingWindowFromWorkspaces = false;
             },
             removeWindowFromOtherWorkspaces: (win) => {
                 each(this.appLists, (appList) => {
@@ -284,8 +248,10 @@ class GroupedWindowListApplet extends Applet.Applet {
                     }
                     appList.windowRemoved(appList.metaWorkspace, win);
                 });
+                this.state.removingWindowFromWorkspaces = false;
             },
             refreshCurrentAppList: () => this.refreshCurrentAppList(),
+            refreshAllAppLists: () => this.refreshAllAppLists(),
             getCurrentAppList: () => this.getCurrentAppList(),
             clearDragPlaceholder: () => this.clearDragPlaceholder(),
             getAutoStartApps: () => this.getAutoStartApps(),
@@ -301,7 +267,7 @@ class GroupedWindowListApplet extends Applet.Applet {
             getFavorites: () => this.pinnedFavorites._favorites,
             cycleWindows: (e, source) => this.handleScroll(e, source),
             openAbout: () => this.openAbout(),
-            configureApplet: () => this.configureApplet()
+            configureApplet: () => this.configureApplet(),
         });
 
         this.settings = new AppletSettings(this.state.settings, metadata.uuid, instance_id);
@@ -331,6 +297,7 @@ class GroupedWindowListApplet extends Applet.Applet {
         this.signals.connect(global.screen, 'window-skip-taskbar-changed', (...args) => this.onWindowSkipTaskbarChanged(...args));
         this.signals.connect(global.display, 'window-marked-urgent', (...args) => this.updateAttentionState(...args));
         this.signals.connect(global.display, 'window-demands-attention', (...args) => this.updateAttentionState(...args));
+        this.signals.connect(global.display, 'window-created', (...args) => this.onWindowCreated(...args));
         this.signals.connect(global.settings, 'changed::panel-edit-mode', (...args) => this.on_panel_edit_mode_changed(...args));
         this.signals.connect(Main.overview, 'showing', (...args) => this.onOverviewShow(...args));
         this.signals.connect(Main.overview, 'hiding', (...args) => this.onOverviewHide(...args));
@@ -341,18 +308,12 @@ class GroupedWindowListApplet extends Applet.Applet {
 
     bindSettings() {
         let settingsProps = [
-            {key: 'show-pinned', value: 'showPinned', cb: this.refreshCurrentAppList},
-            {key: 'show-alerts', value: 'showAlerts', cb: this.updateAttentionState},
             {key: 'group-apps', value: 'groupApps', cb: this.refreshCurrentAppList},
             {key: 'enable-app-button-dragging', value: 'enableDragging', cb: null},
-            {key: 'pinOnDrag', value: 'pinOnDrag', cb: null},
             {key: 'launcher-animation-effect', value: 'launcherAnimationEffect', cb: null},
             {key: 'pinned-apps', value: 'pinnedApps', cb: null},
             {key: 'middle-click-action', value: 'middleClickAction', cb: null},
             {key: 'left-click-action', value: 'leftClickAction', cb: null},
-            {key: 'show-apps-order-hotkey', value: 'showAppsOrderHotkey', cb: this.bindAppKeys},
-            {key: 'show-apps-order-timeout', value: 'showAppsOrderTimeout', cb: null},
-            {key: 'cycleMenusHotkey', value: 'cycleMenusHotkey', cb: this.bindAppKeys},
             {key: 'enable-hover-peek', value: 'enablePeek', cb: null},
             {key: 'onclick-thumbnails', value: 'onClickThumbs', cb: null},
             {key: 'hover-peek-opacity', value: 'peekOpacity', cb: null},
@@ -373,14 +334,11 @@ class GroupedWindowListApplet extends Applet.Applet {
             {key: 'number-display', value: 'numDisplay', cb: this.updateWindowNumberState},
             {key: 'title-display', value: 'titleDisplay', cb: this.updateTitleDisplay},
             {key: 'scroll-behavior', value: 'scrollBehavior', cb: null},
-            {key: 'icon-spacing', value: 'iconSpacing', cb: this.updateSpacing},
             {key: 'show-recent', value: 'showRecent', cb: null},
             {key: 'autostart-menu-item', value: 'autoStart', cb: null},
             {key: 'launch-new-instance-menu-item', value: 'launchNewInstance', cb: null},
             {key: 'monitor-move-all-windows', value: 'monitorMoveAllWindows', cb: null},
-            {key: 'system-favorites', value: 'systemFavorites', cb: this.updateFavorites},
-            {key: 'show-all-workspaces', value: 'showAllWorkspaces', cb: this.refreshAllAppLists},
-            {key: 'list-monitor-windows', value: 'listMonitorWindows', cb: this.handleMonitorWindowsPrefsChange}
+            {key: 'show-all-workspaces', value: 'showAllWorkspaces', cb: this.refreshAllAppLists}
         ];
 
         for (let i = 0, len = settingsProps.length; i < len; i++) {
@@ -398,8 +356,6 @@ class GroupedWindowListApplet extends Applet.Applet {
         if (this.state.appletReady && this.state.panelEditMode) {
             return;
         }
-        this.bindAppKeys();
-        this.updateSpacing();
         this.state.set({appletReady: true});
     }
 
@@ -408,10 +364,8 @@ class GroupedWindowListApplet extends Applet.Applet {
             return;
         }
 
-        if (this.state.settings.listMonitorWindows) {
-            this.numberOfMonitors = null;
-            this.updateMonitorWatchlist();
-        }
+        this.numberOfMonitors = null;
+        this.updateMonitorWatchlist();
 
         if (instance && instance.instance_id === this.instance_id) {
             this.onSwitchWorkspace();
@@ -424,8 +378,8 @@ class GroupedWindowListApplet extends Applet.Applet {
         this.state.set({panelEditMode: !this.state.panelEditMode});
         each(this.appLists, (workspace) => {
             each(workspace.appList, (appGroup) => {
-                appGroup.hoverMenu.actor.reactive = !this.state.panelEditMode;
-                appGroup.rightClickMenu.actor.reactive = !this.state.panelEditMode;
+                if (appGroup.hoverMenu) appGroup.hoverMenu.actor.reactive = !this.state.panelEditMode;
+                if (appGroup.rightClickMenu) appGroup.rightClickMenu.actor.reactive = !this.state.panelEditMode;
                 appGroup.actor.reactive = !this.state.panelEditMode;
             });
         });
@@ -450,7 +404,6 @@ class GroupedWindowListApplet extends Applet.Applet {
 
     on_applet_removed_from_panel() {
         this.state.set({willUnmount: true});
-        this.unbindAppKeys();
         this.signals.disconnectAllSignals();
         for (let i = 0, len = this.appLists.length; i < len; i++) {
             if (this.appLists[i]) {
@@ -474,72 +427,15 @@ class GroupedWindowListApplet extends Applet.Applet {
     }
 
     onWindowMonitorChanged(screen, metaWindow, metaWorkspace) {
-        if (this.state.settings.listMonitorWindows
-            && this.state.monitorWatchList.length !== this.numberOfMonitors) {
+        if (this.state.monitorWatchList.length !== this.numberOfMonitors) {
             let appList = this.getCurrentAppList();
             appList.windowRemoved(metaWorkspace, metaWindow);
             appList.windowAdded(metaWorkspace, metaWindow);
         }
     }
 
-    bindAppKeys() {
-        this.unbindAppKeys();
-
-        for (let i = 1; i < 10; i++) {
-            this.bindAppKey(i);
-        }
-        Main.keybindingManager.addHotKey('launch-show-apps-order', this.state.settings.showAppsOrderHotkey, () =>
-            this.showAppsOrder()
-        );
-        Main.keybindingManager.addHotKey('launch-cycle-menus', this.state.settings.cycleMenusHotkey, () =>
-            this.cycleMenus()
-        );
-    }
-
-    unbindAppKeys() {
-        for (let i = 1; i < 10; i++) {
-            Main.keybindingManager.removeHotKey('launch-app-key-' + i);
-            Main.keybindingManager.removeHotKey('launch-new-app-key-' + i);
-        }
-        Main.keybindingManager.removeHotKey('launch-show-apps-order');
-        Main.keybindingManager.removeHotKey('launch-cycle-menus');
-    }
-
-    bindAppKey(i) {
-        Main.keybindingManager.addHotKey('launch-app-key-' + i, '<Super>' + i, () => this.onAppKeyPress(i));
-        Main.keybindingManager.addHotKey('launch-new-app-key-' + i, '<Super><Shift>' + i, () =>
-            this.onNewAppKeyPress(i)
-        );
-    }
-
-    onAppKeyPress(number) {
-        this.getCurrentAppList().onAppKeyPress(number);
-    }
-
-    onNewAppKeyPress(number) {
-        this.getCurrentAppList().onNewAppKeyPress(number);
-    }
-
-    showAppsOrder() {
-        this.getCurrentAppList().showAppsOrder();
-    }
-
-    cycleMenus() {
-        this.getCurrentAppList().cycleMenus();
-    }
-
     handleMonitorWindowsPrefsChange(value) {
-        let instances = Main.AppletManager.getRunningInstancesForUuid(this.state.uuid);
-        for (let i = 0; i < instances.length; i++) {
-            if (!instances[i]) {
-                continue;
-            }
-            instances[i].updateMonitorWatchlist();
-            if (instances[i].panel.monitorIndex !== this.panel.monitorIndex) {
-                instances[i].state.settings.listMonitorWindows = this.state.settings.listMonitorWindows;
-            }
-            instances[i].refreshCurrentAppList();
-        }
+        
     }
 
     updateMonitorWatchlist() {
@@ -596,7 +492,7 @@ class GroupedWindowListApplet extends Applet.Applet {
     updateThumbnailSize() {
         each(this.appLists, (workspace) => {
             each(workspace.appList, (appGroup) => {
-                appGroup.hoverMenu.updateThumbnailSize();
+                if (appGroup.hoverMenu) appGroup.hoverMenu.updateThumbnailSize();
             });
         });
     }
@@ -611,12 +507,6 @@ class GroupedWindowListApplet extends Applet.Applet {
         });
     }
 
-    updateSpacing() {
-        each(this.appLists, (workspace) => {
-            workspace.updateSpacing();
-        });
-    }
-
     updateWindowNumberState() {
         each(this.appLists, (workspace) => {
             workspace.calcAllWindowNumbers();
@@ -624,14 +514,17 @@ class GroupedWindowListApplet extends Applet.Applet {
     }
 
     updateAttentionState(display, window) {
-        if (!this.state.settings.showAlerts) {
-            return false;
-        }
         each(this.appLists, (workspace) => {
             workspace.updateAttentionState(display, window);
         });
     }
 
+    onWindowCreated(display, window) {
+        each(this.appLists, (workspace) => {
+            workspace.windowAdded(window.get_workspace(), window);
+        });
+    }
+
     updateVerticalThumbnailState() {
         each(this.appLists, (workspace) => {
             each(workspace.appList, (appGroup) => {
@@ -647,13 +540,16 @@ class GroupedWindowListApplet extends Applet.Applet {
             || this.state.lastTitleDisplay === TitleDisplay.None) {
             this.refreshCurrentAppList();
         }
-        let appList = this.getCurrentAppList().appList;
-        each(appList, (appGroup) => {
-            if (titleDisplay === TitleDisplay.Focused) {
-                appGroup.hideLabel(false);
-            }
-            appGroup.handleTitleDisplayChange();
+
+        each(this.appLists, (workspace) => {
+            each(workspace.appList, (appGroup) => {
+                if (titleDisplay === TitleDisplay.Focused) {
+                    appGroup.hideLabel(false);
+                }
+                appGroup.handleTitleDisplayChange();
+            });
         });
+
         this.state.set({lastTitleDisplay: titleDisplay});
     }
 
@@ -663,9 +559,6 @@ class GroupedWindowListApplet extends Applet.Applet {
             for (let i = 0, len = specialApps.length; i < len; i++) {
                 if (specialApps[i].wmClass === wmClass) {
                     app = this.appSystem.lookup_app(specialApps[i].id);
-                    if (!app) {
-                        app = this.appSystem.lookup_settings_app(specialApps[i].id);
-                    }
                     if (app) {
                         app.wmClass = wmClass;
                     }
@@ -800,6 +693,7 @@ class GroupedWindowListApplet extends Applet.Applet {
         let appList = this.appLists[this.state.currentWs];
         let children = appList.actor.get_children();
         let windowPos = children.indexOf(source.actor);
+        let isForeign = typeof source.groupState === 'undefined';
 
         let pos = 0;
 
@@ -819,12 +713,7 @@ class GroupedWindowListApplet extends Applet.Applet {
                 if (this.state.dragPlaceholder) {
                     this.state.dragPlaceholder.animateOutAndDestroy();
                     this.state.animatingPlaceholdersCount++;
-                    this.state.dragPlaceholder.actor.connect(
-                        'destroy',
-                        () => {
-                            this.state.animatingPlaceholdersCount--;
-                        }
-                    );
+                    this.state.dragPlaceholder.actor.connect('destroy', () => this.state.animatingPlaceholdersCount--);
                 }
                 this.state.dragPlaceholder = null;
 
@@ -842,15 +731,19 @@ class GroupedWindowListApplet extends Applet.Applet {
                 fadeIn = true;
             }
 
-            let childWidth = source.actor.width;
-            let childHeight = source.actor.height;
+            let iconSize = this.getPanelIconSize();
             this.state.dragPlaceholder = new DND.GenericDragPlaceholderItem();
-            this.state.dragPlaceholder.child.width = childWidth;
-            this.state.dragPlaceholder.child.height = childHeight;
-            appList.actor.insert_child_at_index(
-                this.state.dragPlaceholder.actor,
-                this.state.dragPlaceholderPos
-            );
+            this.state.dragPlaceholder.child.width = iconSize;
+            this.state.dragPlaceholder.child.height = iconSize;
+
+            // For menu items, don't insert actors at specific indices,
+            // they will get attached to the end of the list anyway.
+            if (!isForeign) {
+                appList.actor.insert_child_at_index(
+                    this.state.dragPlaceholder.actor,
+                    this.state.dragPlaceholderPos
+                );
+            }
 
             if (fadeIn) this.state.dragPlaceholder.animateIn();
         }
@@ -895,6 +788,7 @@ class GroupedWindowListApplet extends Applet.Applet {
         Meta.later_add(Meta.LaterType.BEFORE_REDRAW, () => {
             // Move the button
             currentAppList.actor.set_child_at_index(source.actor, pos);
+            currentAppList.updateAppGroupIndexes();
             // Refresh the group's thumbnails so hoverMenu is aware of the position change
             // In the case of dragging a group that has a delay before Cinnamon can grab its
             // thumbnail texture, e.g., LibreOffice, defer the refresh.
@@ -904,8 +798,7 @@ class GroupedWindowListApplet extends Applet.Applet {
 
 
             // Handle favoriting if pin on drag is enabled
-            if (this.state.settings.pinOnDrag
-                && !source.groupState.app.is_window_backed()) {
+            if (!source.groupState.app.is_window_backed()) {
                 let opts = {
                     appId: source.groupState.appId,
                     app: source.groupState.app,
@@ -914,8 +807,6 @@ class GroupedWindowListApplet extends Applet.Applet {
                 let refFav = findIndex(this.pinnedFavorites._favorites, (favorite) => favorite.id === source.groupState.appId);
                 if (refFav > -1) {
                     this.pinnedFavorites.moveFavoriteToPos(opts);
-                } else if (this.state.settings.pinOnDrag) {
-                    this.pinnedFavorites.addFavorite(opts);
                 }
             }
 
